@@ -22,7 +22,7 @@ extern size_t onig_region_memsize(const struct re_registers *regs);
 
 #include <stdbool.h>
 
-#define STRSCAN_VERSION "3.0.0"
+#define STRSCAN_VERSION "3.0.5"
 
 /* =======================================================================
                          Data Type Definitions
@@ -435,23 +435,20 @@ strscan_get_pos(VALUE self)
  *
  * In short, it's a 0-based index into the string.
  *
- *   s = StringScanner.new("abcädeföghi")
- *   s.charpos           # -> 0
- *   s.scan_until(/ä/)   # -> "abcä"
- *   s.pos               # -> 5
- *   s.charpos           # -> 4
+ *   s = StringScanner.new("abc\u00e4def\u00f6ghi")
+ *   s.charpos                # -> 0
+ *   s.scan_until(/\u00e4/)   # -> "abc\u00E4"
+ *   s.pos                    # -> 5
+ *   s.charpos                # -> 4
  */
 static VALUE
 strscan_get_charpos(VALUE self)
 {
     struct strscanner *p;
-    VALUE substr;
 
     GET_SCANNER(self, p);
 
-    substr = rb_funcall(p->str, id_byteslice, 2, INT2FIX(0), LONG2NUM(p->curr));
-
-    return rb_str_length(substr);
+    return LONG2NUM(rb_enc_strlen(S_PBEG(p), CURPTR(p), rb_enc_get(p->str)));
 }
 
 /*
@@ -984,7 +981,7 @@ strscan_unscan(VALUE self)
 }
 
 /*
- * Returns +true+ iff the scan pointer is at the beginning of the line.
+ * Returns +true+ if and only if the scan pointer is at the beginning of the line.
  *
  *   s = StringScanner.new("test\ntest\n")
  *   s.bol?           # => true
@@ -1037,7 +1034,7 @@ strscan_empty_p(VALUE self)
 }
 
 /*
- * Returns true iff there is more data in the string.  See #eos?.
+ * Returns true if and only if there is more data in the string.  See #eos?.
  * This method is obsolete; use #eos? instead.
  *
  *   s = StringScanner.new('test string')
@@ -1054,7 +1051,7 @@ strscan_rest_p(VALUE self)
 }
 
 /*
- * Returns +true+ iff the last match was successful.
+ * Returns +true+ if and only if the last match was successful.
  *
  *   s = StringScanner.new('test string')
  *   s.match?(/\w+/)     # => 4
@@ -1461,6 +1458,54 @@ strscan_fixed_anchor_p(VALUE self)
     return p->fixed_anchor_p ? Qtrue : Qfalse;
 }
 
+typedef struct {
+    VALUE self;
+    VALUE captures;
+} named_captures_data;
+
+static int
+named_captures_iter(const OnigUChar *name,
+                    const OnigUChar *name_end,
+                    int back_num,
+                    int *back_refs,
+                    OnigRegex regex,
+                    void *arg)
+{
+    named_captures_data *data = arg;
+
+    VALUE key = rb_str_new((const char *)name, name_end - name);
+    VALUE value = RUBY_Qnil;
+    int i;
+    for (i = 0; i < back_num; i++) {
+        value = strscan_aref(data->self, INT2NUM(back_refs[i]));
+    }
+    rb_hash_aset(data->captures, key, value);
+    return 0;
+}
+
+/*
+ * call-seq:
+ *   scanner.named_captures -> hash
+ *
+ * Returns a hash of string variables matching the regular expression.
+ *
+ *   scan = StringScanner.new('foobarbaz')
+ *   scan.match?(/(?<f>foo)(?<r>bar)(?<z>baz)/)
+ *   scan.named_captures # -> {"f"=>"foo", "r"=>"bar", "z"=>"baz"}
+ */
+static VALUE
+strscan_named_captures(VALUE self)
+{
+    struct strscanner *p;
+    GET_SCANNER(self, p);
+    named_captures_data data;
+    data.self = self;
+    data.captures = rb_hash_new();
+    onig_foreach_name(RREGEXP_PTR(p->regex), named_captures_iter, &data);
+
+    return data.captures;
+}
+
 /* =======================================================================
                               Ruby Interface
    ======================================================================= */
@@ -1471,6 +1516,8 @@ strscan_fixed_anchor_p(VALUE self)
  * StringScanner provides for lexical scanning operations on a String.  Here is
  * an example of its usage:
  *
+ *   require 'strscan'
+ *
  *   s = StringScanner.new('This is an example string')
  *   s.eos?               # -> false
  *
@@ -1537,7 +1584,7 @@ strscan_fixed_anchor_p(VALUE self)
  *
  * === Finding Where we Are
  *
- * - #beginning_of_line? (#bol?)
+ * - #beginning_of_line? (<tt>#bol?</tt>)
  * - #eos?
  * - #rest?
  * - #rest_size
@@ -1554,13 +1601,13 @@ strscan_fixed_anchor_p(VALUE self)
  * - #matched
  * - #matched?
  * - #matched_size
- * - []
+ * - <tt>#[]</tt>
  * - #pre_match
  * - #post_match
  *
  * === Miscellaneous
  *
- * - <<
+ * - <tt><<</tt>
  * - #concat
  * - #string
  * - #string=
@@ -1653,4 +1700,6 @@ Init_strscan(void)
     rb_define_method(StringScanner, "inspect",     strscan_inspect,     0);
 
     rb_define_method(StringScanner, "fixed_anchor?", strscan_fixed_anchor_p, 0);
+
+    rb_define_method(StringScanner, "named_captures", strscan_named_captures, 0);
 }
