@@ -6,10 +6,12 @@
 #define SET_MACHINE_STACK_END(p) __asm__ __volatile__ ("movq\t%%rsp, %0" : "=r" (*(p)))
 #elif defined(__i386) && defined(__GNUC__)
 #define SET_MACHINE_STACK_END(p) __asm__ __volatile__ ("movl\t%%esp, %0" : "=r" (*(p)))
-#elif (defined(__powerpc__) || defined(__powerpc64__)) && defined(__GNUC__) && !defined(_AIX)
+#elif (defined(__powerpc__) || defined(__powerpc64__)) && defined(__GNUC__) && !defined(_AIX) && !defined(__APPLE__) // Not Apple is NEEDED to unbreak ppc64 build on Darwin. Don't ask.
 #define SET_MACHINE_STACK_END(p) __asm__ __volatile__ ("mr\t%0, %%r1" : "=r" (*(p)))
 #elif (defined(__powerpc__) || defined(__powerpc64__)) && defined(__GNUC__) && defined(_AIX)
 #define SET_MACHINE_STACK_END(p) __asm__ __volatile__ ("mr %0,1" : "=r" (*(p)))
+#elif defined(__POWERPC__) && defined(__APPLE__) // Darwin ppc and ppc64
+#define SET_MACHINE_STACK_END(p) __asm__ volatile("mr %0, r1" : "=r" (*(p)))
 #elif defined(__aarch64__) && defined(__GNUC__)
 #define SET_MACHINE_STACK_END(p) __asm__ __volatile__ ("mov\t%0, sp" : "=r" (*(p)))
 #else
@@ -20,9 +22,9 @@ NOINLINE(void rb_gc_set_stack_end(VALUE **stack_end_p));
 
 #define RB_GC_SAVE_MACHINE_CONTEXT(th)				\
     do {							\
-	FLUSH_REGISTER_WINDOWS;					\
-	setjmp((th)->ec->machine.regs);				\
-	SET_MACHINE_STACK_END(&(th)->ec->machine.stack_end);	\
+        FLUSH_REGISTER_WINDOWS;					\
+        setjmp((th)->ec->machine.regs);				\
+        SET_MACHINE_STACK_END(&(th)->ec->machine.stack_end);	\
     } while (0)
 
 /* for GC debug */
@@ -44,13 +46,13 @@ static inline void
 rb_gc_debug_body(const char *mode, const char *msg, int st, void *ptr)
 {
     if (st == 0) {
-	ruby_gc_debug_indent--;
+        ruby_gc_debug_indent--;
     }
     rb_gc_debug_indent();
     ruby_debug_printf("%s: %s %s (%p)\n", mode, st ? "->" : "<-", msg, ptr);
 
     if (st) {
-	ruby_gc_debug_indent++;
+        ruby_gc_debug_indent++;
     }
 
     fflush(stdout);
@@ -88,9 +90,9 @@ rb_gc_debug_body(const char *mode, const char *msg, int st, void *ptr)
 RUBY_EXTERN int ruby_stack_grow_direction;
 int ruby_get_stack_grow_direction(volatile VALUE *addr);
 # define stack_growup_p(x) (			\
-	(ruby_stack_grow_direction ?		\
-	 ruby_stack_grow_direction :		\
-	 ruby_get_stack_grow_direction(x)) > 0)
+        (ruby_stack_grow_direction ?		\
+         ruby_stack_grow_direction :		\
+         ruby_get_stack_grow_direction(x)) > 0)
 # define STACK_UPPER(x, a, b) (stack_growup_p(x) ? (a) : (b))
 #endif
 
@@ -112,12 +114,14 @@ int ruby_get_stack_grow_direction(volatile VALUE *addr);
 #define IS_STACK_DIR_UPPER() STACK_DIR_UPPER(1,0)
 
 const char *rb_obj_info(VALUE obj);
-const char *rb_raw_obj_info(char *buff, const int buff_size, VALUE obj);
+const char *rb_raw_obj_info(char *const buff, const size_t buff_size, VALUE obj);
 
 VALUE rb_gc_disable_no_rest(void);
 
 struct rb_thread_struct;
 
+size_t rb_size_pool_slot_size(unsigned char pool_id);
+
 RUBY_SYMBOL_EXPORT_BEGIN
 
 /* exports for objspace module */
@@ -136,6 +140,8 @@ void rb_objspace_each_objects_without_setup(
     int (*callback)(void *, void *, size_t, void *),
     void *data);
 
+size_t rb_gc_obj_slot_size(VALUE obj);
+
 RUBY_SYMBOL_EXPORT_END
 
 #endif /* RUBY_GC_H */
