@@ -1,6 +1,6 @@
 # frozen_string_literal: true
-require 'rubygems/installer_test_case'
-require 'rubygems/uninstaller'
+require_relative "installer_test_case"
+require "rubygems/uninstaller"
 
 class TestGemUninstaller < Gem::InstallerTestCase
   def setup
@@ -23,14 +23,14 @@ class TestGemUninstaller < Gem::InstallerTestCase
   end
 
   def test_initialize_expand_path
-    FileUtils.mkdir_p 'foo/bar'
-    uninstaller = Gem::Uninstaller.new nil, :install_dir => 'foo//bar'
+    FileUtils.mkdir_p "foo/bar"
+    uninstaller = Gem::Uninstaller.new nil, :install_dir => "foo//bar"
 
     assert_match %r{foo/bar$}, uninstaller.instance_variable_get(:@gem_home)
   end
 
   def test_ask_if_ok
-    c = util_spec 'c'
+    c = util_spec "c"
 
     uninstaller = Gem::Uninstaller.new nil
 
@@ -60,8 +60,8 @@ class TestGemUninstaller < Gem::InstallerTestCase
   def test_remove_executables_force_keep
     uninstaller = Gem::Uninstaller.new nil, :executables => false
 
-    executable = File.join Gem.bindir(@user_spec.base_dir), 'executable'
-    assert File.exist?(executable), 'executable not written'
+    executable = File.join Gem.bindir(@user_spec.base_dir), "executable"
+    assert File.exist?(executable), "executable not written"
 
     use_ui @ui do
       uninstaller.remove_executables @user_spec
@@ -75,8 +75,8 @@ class TestGemUninstaller < Gem::InstallerTestCase
   def test_remove_executables_force_remove
     uninstaller = Gem::Uninstaller.new nil, :executables => true
 
-    executable = File.join Gem.bindir(@user_spec.base_dir), 'executable'
-    assert File.exist?(executable), 'executable not written'
+    executable = File.join Gem.bindir(@user_spec.base_dir), "executable"
+    assert File.exist?(executable), "executable not written"
 
     use_ui @ui do
       uninstaller.remove_executables @user_spec
@@ -94,14 +94,14 @@ class TestGemUninstaller < Gem::InstallerTestCase
       uninstaller.remove_executables @user_spec
     end
 
-    exec_path = File.join Gem.user_dir, 'bin', 'executable'
-    refute File.exist?(exec_path), 'exec still exists in user bin dir'
+    exec_path = File.join Gem.user_dir, "bin", "executable"
+    refute File.exist?(exec_path), "exec still exists in user bin dir"
 
     assert_equal "Removing executable\n", @ui.output
   end
 
   def test_remove_executables_user_format
-    Gem::Installer.exec_format = 'foo-%s-bar'
+    Gem::Installer.exec_format = "foo-%s-bar"
 
     uninstaller = Gem::Uninstaller.new nil, :executables => true, :format_executable => true
 
@@ -109,8 +109,8 @@ class TestGemUninstaller < Gem::InstallerTestCase
       uninstaller.remove_executables @user_spec
     end
 
-    exec_path = File.join Gem.user_dir, 'bin', 'foo-executable-bar'
-    assert_equal false, File.exist?(exec_path), 'removed exec from bin dir'
+    exec_path = File.join Gem.user_dir, "bin", "foo-executable-bar"
+    assert_equal false, File.exist?(exec_path), "removed exec from bin dir"
 
     assert_equal "Removing foo-executable-bar\n", @ui.output
   ensure
@@ -118,7 +118,7 @@ class TestGemUninstaller < Gem::InstallerTestCase
   end
 
   def test_remove_executables_user_format_disabled
-    Gem::Installer.exec_format = 'foo-%s-bar'
+    Gem::Installer.exec_format = "foo-%s-bar"
 
     uninstaller = Gem::Uninstaller.new nil, :executables => true
 
@@ -126,8 +126,8 @@ class TestGemUninstaller < Gem::InstallerTestCase
       uninstaller.remove_executables @user_spec
     end
 
-    exec_path = File.join Gem.user_dir, 'bin', 'executable'
-    refute File.exist?(exec_path), 'removed exec from bin dir'
+    exec_path = File.join Gem.user_dir, "bin", "executable"
+    refute File.exist?(exec_path), "removed exec from bin dir"
 
     assert_equal "Removing executable\n", @ui.output
   ensure
@@ -171,7 +171,7 @@ class TestGemUninstaller < Gem::InstallerTestCase
   end
 
   def test_remove_plugins
-    write_file File.join(@tempdir, 'lib', 'rubygems_plugin.rb') do |io|
+    write_file File.join(@tempdir, "lib", "rubygems_plugin.rb") do |io|
       io.write "puts __FILE__"
     end
 
@@ -179,16 +179,16 @@ class TestGemUninstaller < Gem::InstallerTestCase
 
     Gem::Installer.at(Gem::Package.build(@spec), :force => true).install
 
-    plugin_path = File.join Gem.plugindir, 'a_plugin.rb'
-    assert File.exist?(plugin_path), 'plugin not written'
+    plugin_path = File.join Gem.plugindir, "a_plugin.rb"
+    assert File.exist?(plugin_path), "plugin not written"
 
     Gem::Uninstaller.new(nil).remove_plugins @spec
 
-    refute File.exist?(plugin_path), 'plugin not removed'
+    refute File.exist?(plugin_path), "plugin not removed"
   end
 
   def test_remove_plugins_with_install_dir
-    write_file File.join(@tempdir, 'lib', 'rubygems_plugin.rb') do |io|
+    write_file File.join(@tempdir, "lib", "rubygems_plugin.rb") do |io|
       io.write "puts __FILE__"
     end
 
@@ -196,17 +196,17 @@ class TestGemUninstaller < Gem::InstallerTestCase
 
     Gem::Installer.at(Gem::Package.build(@spec), :force => true).install
 
-    plugin_path = File.join Gem.plugindir, 'a_plugin.rb'
-    assert File.exist?(plugin_path), 'plugin not written'
+    plugin_path = File.join Gem.plugindir, "a_plugin.rb"
+    assert File.exist?(plugin_path), "plugin not written"
 
     Dir.mkdir "#{@gemhome}2"
     Gem::Uninstaller.new(nil, :install_dir => "#{@gemhome}2").remove_plugins @spec
 
-    assert File.exist?(plugin_path), 'plugin unintentionally removed'
+    assert File.exist?(plugin_path), "plugin unintentionally removed"
   end
 
   def test_regenerate_plugins_for
-    write_file File.join(@tempdir, 'lib', 'rubygems_plugin.rb') do |io|
+    write_file File.join(@tempdir, "lib", "rubygems_plugin.rb") do |io|
       io.write "puts __FILE__"
     end
 
@@ -214,13 +214,13 @@ class TestGemUninstaller < Gem::InstallerTestCase
 
     Gem::Installer.at(Gem::Package.build(@spec), :force => true).install
 
-    plugin_path = File.join Gem.plugindir, 'a_plugin.rb'
-    assert File.exist?(plugin_path), 'plugin not written'
+    plugin_path = File.join Gem.plugindir, "a_plugin.rb"
+    assert File.exist?(plugin_path), "plugin not written"
 
     FileUtils.rm plugin_path
     Gem::Uninstaller.new(nil).regenerate_plugins_for @spec, Gem.plugindir
 
-    assert File.exist?(plugin_path), 'plugin not regenerated'
+    assert File.exist?(plugin_path), "plugin not regenerated"
   end
 
   def test_path_ok_eh
@@ -234,7 +234,7 @@ class TestGemUninstaller < Gem::InstallerTestCase
 
     @spec.loaded_from = @spec.loaded_from.gsub @spec.full_name, '\&-legacy'
     @spec.internal_init # blow out cache. but why did ^^ depend on cache?
-    @spec.platform = 'legacy'
+    @spec.platform = "legacy"
 
     assert_equal true, uninstaller.path_ok?(@gemhome, @spec)
   end
@@ -248,16 +248,16 @@ class TestGemUninstaller < Gem::InstallerTestCase
   def test_uninstall
     uninstaller = Gem::Uninstaller.new @spec.name, :executables => true
 
-    gem_dir = File.join @gemhome, 'gems', @spec.full_name
+    gem_dir = File.join @gemhome, "gems", @spec.full_name
 
     Gem.pre_uninstall do
       sleep(0.1) if win_platform?
-      assert File.exist?(gem_dir), 'gem_dir should exist'
+      assert File.exist?(gem_dir), "gem_dir should exist"
     end
 
     Gem.post_uninstall do
       sleep(0.1) if win_platform?
-      refute File.exist?(gem_dir), 'gem_dir should not exist'
+      refute File.exist?(gem_dir), "gem_dir should not exist"
     end
 
     uninstaller.uninstall
@@ -269,7 +269,7 @@ class TestGemUninstaller < Gem::InstallerTestCase
   end
 
   def test_uninstall_default_gem
-    spec = new_default_spec 'default', '2'
+    spec = new_default_spec "default", "2"
 
     install_default_gems spec
 
@@ -281,28 +281,35 @@ class TestGemUninstaller < Gem::InstallerTestCase
 
     lines = @ui.output.split("\n")
 
-    assert_equal 'Gem default-2 cannot be uninstalled because it is a default gem', lines.shift
+    assert_equal "Gem default-2 cannot be uninstalled because it is a default gem", lines.shift
   end
 
   def test_uninstall_default_gem_with_same_version
-    default_spec = new_default_spec 'default', '2'
+    default_spec = new_default_spec "default", "2"
     install_default_gems default_spec
 
-    spec = util_spec 'default', '2'
+    spec = util_spec "default", "2"
     install_gem spec
 
     Gem::Specification.reset
 
     uninstaller = Gem::Uninstaller.new spec.name, :executables => true
 
-    uninstaller.uninstall
-
+    ui = Gem::MockGemUi.new "1\ny\n"
+    use_ui ui do
+      uninstaller.uninstall
+    end
+    expected = "Successfully uninstalled default-2\n" \
+      "There was both a regular copy and a default copy of default-2. The " \
+      "regular copy was successfully uninstalled, but the default copy " \
+      "was left around because default gems can't be removed.\n"
+    assert_equal expected, ui.output
     assert_path_not_exist spec.gem_dir
   end
 
   def test_uninstall_extension
-    @spec.extensions << 'extconf.rb'
-    write_file File.join(@tempdir, 'extconf.rb') do |io|
+    @spec.extensions << "extconf.rb"
+    write_file File.join(@tempdir, "extconf.rb") do |io|
       io.write <<-RUBY
 require 'mkmf'
 create_makefile '#{@spec.name}'
@@ -318,7 +325,7 @@ create_makefile '#{@spec.name}'
       installer.install
     end
 
-    assert_path_exist @spec.extension_dir, 'sanity check'
+    assert_path_exist @spec.extension_dir, "sanity check"
 
     uninstaller = Gem::Uninstaller.new @spec.name, :executables => true
     uninstaller.uninstall
@@ -327,7 +334,7 @@ create_makefile '#{@spec.name}'
   end
 
   def test_uninstall_nonexistent
-    uninstaller = Gem::Uninstaller.new 'bogus', :executables => true
+    uninstaller = Gem::Uninstaller.new "bogus", :executables => true
 
     e = assert_raise Gem::InstallError do
       uninstaller.uninstall
@@ -337,17 +344,17 @@ create_makefile '#{@spec.name}'
   end
 
   def test_uninstall_not_ok
-    quick_gem 'z' do |s|
+    quick_gem "z" do |s|
       s.add_runtime_dependency @spec.name
     end
 
     uninstaller = Gem::Uninstaller.new @spec.name
 
-    gem_dir = File.join @gemhome, 'gems', @spec.full_name
-    executable = File.join @gemhome, 'bin', 'executable'
+    gem_dir = File.join @gemhome, "gems", @spec.full_name
+    executable = File.join @gemhome, "bin", "executable"
 
-    assert File.exist?(gem_dir),    'gem_dir must exist'
-    assert File.exist?(executable), 'executable must exist'
+    assert File.exist?(gem_dir),    "gem_dir must exist"
+    assert File.exist?(executable), "executable must exist"
 
     ui = Gem::MockGemUi.new "n\n"
 
@@ -357,15 +364,15 @@ create_makefile '#{@spec.name}'
       end
     end
 
-    assert File.exist?(gem_dir),    'gem_dir must still exist'
-    assert File.exist?(executable), 'executable must still exist'
+    assert File.exist?(gem_dir),    "gem_dir must still exist"
+    assert File.exist?(executable), "executable must still exist"
   end
 
   def test_uninstall_user_install
-    @user_spec = Gem::Specification.find_by_name 'b'
+    @user_spec = Gem::Specification.find_by_name "b"
 
     uninstaller = Gem::Uninstaller.new(@user_spec.name,
-                                       :executables  => true,
+                                       :executables => true,
                                        :user_install => true)
 
     gem_dir = File.join @user_spec.gem_dir
@@ -407,9 +414,9 @@ create_makefile '#{@spec.name}'
   def test_uninstall_selection
     util_make_gems
 
-    list = Gem::Specification.find_all_by_name 'a'
+    list = Gem::Specification.find_all_by_name "a"
 
-    uninstaller = Gem::Uninstaller.new 'a'
+    uninstaller = Gem::Uninstaller.new "a"
 
     ui = Gem::MockGemUi.new "1\ny\n"
 
@@ -417,39 +424,39 @@ create_makefile '#{@spec.name}'
       uninstaller.uninstall
     end
 
-    updated_list = Gem::Specification.find_all_by_name('a')
+    updated_list = Gem::Specification.find_all_by_name("a")
     assert_equal list.length - 1, updated_list.length
 
-    assert_match ' 1. a-1',          ui.output
-    assert_match ' 2. a-2',          ui.output
-    assert_match ' 3. a-3.a',        ui.output
-    assert_match ' 4. All versions', ui.output
-    assert_match 'uninstalled a-1',  ui.output
+    assert_match " 1. a-1",          ui.output
+    assert_match " 2. a-2",          ui.output
+    assert_match " 3. a-3.a",        ui.output
+    assert_match " 4. All versions", ui.output
+    assert_match "uninstalled a-1",  ui.output
   end
 
   def test_uninstall_selection_greater_than_one
     util_make_gems
 
-    list = Gem::Specification.find_all_by_name('a')
+    list = Gem::Specification.find_all_by_name("a")
 
-    uninstaller = Gem::Uninstaller.new('a')
+    uninstaller = Gem::Uninstaller.new("a")
 
     use_ui Gem::MockGemUi.new("2\ny\n") do
       uninstaller.uninstall
     end
 
-    updated_list = Gem::Specification.find_all_by_name('a')
+    updated_list = Gem::Specification.find_all_by_name("a")
     assert_equal list.length - 1, updated_list.length
   end
 
   def test_uninstall_prompts_about_broken_deps
-    quick_gem 'r', '1' do |s|
-      s.add_dependency 'q', '= 1'
+    quick_gem "r", "1" do |s|
+      s.add_dependency "q", "= 1"
     end
 
-    quick_gem 'q', '1'
+    quick_gem "q", "1"
 
-    un = Gem::Uninstaller.new('q')
+    un = Gem::Uninstaller.new("q")
     ui = Gem::MockGemUi.new("y\n")
 
     use_ui ui do
@@ -468,18 +475,18 @@ create_makefile '#{@spec.name}'
   end
 
   def test_uninstall_only_lists_unsatisfied_deps
-    quick_gem 'r', '1' do |s|
-      s.add_dependency 'q', '~> 1.0'
+    quick_gem "r", "1" do |s|
+      s.add_dependency "q", "~> 1.0"
     end
 
-    quick_gem 'x', '1' do |s|
-      s.add_dependency 'q', '= 1.0'
+    quick_gem "x", "1" do |s|
+      s.add_dependency "q", "= 1.0"
     end
 
-    quick_gem 'q', '1.0'
-    quick_gem 'q', '1.1'
+    quick_gem "q", "1.0"
+    quick_gem "q", "1.1"
 
-    un = Gem::Uninstaller.new('q', :version => "1.0")
+    un = Gem::Uninstaller.new("q", :version => "1.0")
     ui = Gem::MockGemUi.new("y\n")
 
     use_ui ui do
@@ -498,14 +505,14 @@ create_makefile '#{@spec.name}'
   end
 
   def test_uninstall_doesnt_prompt_when_other_gem_satisfies_requirement
-    quick_gem 'r', '1' do |s|
-      s.add_dependency 'q', '~> 1.0'
+    quick_gem "r", "1" do |s|
+      s.add_dependency "q", "~> 1.0"
     end
 
-    quick_gem 'q', '1.0'
-    quick_gem 'q', '1.1'
+    quick_gem "q", "1.0"
+    quick_gem "q", "1.1"
 
-    un = Gem::Uninstaller.new('q', :version => "1.0")
+    un = Gem::Uninstaller.new("q", :version => "1.0")
     ui = Gem::MockGemUi.new("y\n")
 
     use_ui ui do
@@ -518,13 +525,13 @@ create_makefile '#{@spec.name}'
   end
 
   def test_uninstall_doesnt_prompt_when_removing_a_dev_dep
-    quick_gem 'r', '1' do |s|
-      s.add_development_dependency 'q', '= 1.0'
+    quick_gem "r", "1" do |s|
+      s.add_development_dependency "q", "= 1.0"
     end
 
-    quick_gem 'q', '1.0'
+    quick_gem "q", "1.0"
 
-    un = Gem::Uninstaller.new('q', :version => "1.0")
+    un = Gem::Uninstaller.new("q", :version => "1.0")
     ui = Gem::MockGemUi.new("y\n")
 
     use_ui ui do
@@ -537,13 +544,13 @@ create_makefile '#{@spec.name}'
   end
 
   def test_uninstall_doesnt_prompt_and_raises_when_abort_on_dependent_set
-    quick_gem 'r', '1' do |s|
-      s.add_dependency 'q', '= 1'
+    quick_gem "r", "1" do |s|
+      s.add_dependency "q", "= 1"
     end
 
-    quick_gem 'q', '1'
+    quick_gem "q", "1"
 
-    un = Gem::Uninstaller.new('q', :abort_on_dependent => true)
+    un = Gem::Uninstaller.new("q", :abort_on_dependent => true)
     ui = Gem::MockGemUi.new("y\n")
 
     assert_raise Gem::DependencyRemovalException do
@@ -554,13 +561,13 @@ create_makefile '#{@spec.name}'
   end
 
   def test_uninstall_prompt_includes_dep_type
-    quick_gem 'r', '1' do |s|
-      s.add_development_dependency 'q', '= 1'
+    quick_gem "r", "1" do |s|
+      s.add_development_dependency "q", "= 1"
     end
 
-    quick_gem 'q', '1'
+    quick_gem "q", "1"
 
-    un = Gem::Uninstaller.new('q', :check_dev => true)
+    un = Gem::Uninstaller.new("q", :check_dev => true)
     ui = Gem::MockGemUi.new("y\n")
 
     use_ui ui do
@@ -579,17 +586,17 @@ create_makefile '#{@spec.name}'
   end
 
   def test_uninstall_prompt_only_lists_the_dependents_that_prevented_uninstallation
-    quick_gem 'r', '1' do |s|
-      s.add_development_dependency 'q', '= 1'
+    quick_gem "r", "1" do |s|
+      s.add_development_dependency "q", "= 1"
     end
 
-    quick_gem 's', '1' do |s|
-      s.add_dependency 'q', '= 1'
+    quick_gem "s", "1" do |s|
+      s.add_dependency "q", "= 1"
     end
 
-    quick_gem 'q', '1'
+    quick_gem "q", "1"
 
-    un = Gem::Uninstaller.new('q', :check_dev => false)
+    un = Gem::Uninstaller.new("q", :check_dev => false)
     ui = Gem::MockGemUi.new("y\n")
 
     use_ui ui do
@@ -626,42 +633,42 @@ create_makefile '#{@spec.name}'
   end
 
   def test_uninstall_keeps_plugins_up_to_date
-    write_file File.join(@tempdir, 'lib', 'rubygems_plugin.rb') do |io|
+    write_file File.join(@tempdir, "lib", "rubygems_plugin.rb") do |io|
       io.write "puts __FILE__"
     end
 
-    plugin_path = File.join Gem.plugindir, 'a_plugin.rb'
+    plugin_path = File.join Gem.plugindir, "a_plugin.rb"
 
-    @spec.version = '1'
+    @spec.version = "1"
     Gem::Installer.at(Gem::Package.build(@spec), :force => true).install
 
-    refute File.exist?(plugin_path), 'version without plugin installed, but plugin written'
+    refute File.exist?(plugin_path), "version without plugin installed, but plugin written"
 
     @spec.files += %w[lib/rubygems_plugin.rb]
-    @spec.version = '2'
+    @spec.version = "2"
     Gem::Installer.at(Gem::Package.build(@spec), :force => true).install
 
-    assert File.exist?(plugin_path), 'version with plugin installed, but plugin not written'
-    assert_match %r{\Arequire.*a-2/lib/rubygems_plugin\.rb}, File.read(plugin_path), 'written plugin has incorrect content'
+    assert File.exist?(plugin_path), "version with plugin installed, but plugin not written"
+    assert_match %r{\Arequire.*a-2/lib/rubygems_plugin\.rb}, File.read(plugin_path), "written plugin has incorrect content"
 
-    @spec.version = '3'
+    @spec.version = "3"
     Gem::Installer.at(Gem::Package.build(@spec), :force => true).install
 
-    assert File.exist?(plugin_path), 'version with plugin installed, but plugin removed'
-    assert_match %r{\Arequire.*a-3/lib/rubygems_plugin\.rb}, File.read(plugin_path), 'old version installed, but plugin updated'
+    assert File.exist?(plugin_path), "version with plugin installed, but plugin removed"
+    assert_match %r{\Arequire.*a-3/lib/rubygems_plugin\.rb}, File.read(plugin_path), "old version installed, but plugin updated"
 
-    Gem::Uninstaller.new('a', :version => '1', :executables => true).uninstall
+    Gem::Uninstaller.new("a", :version => "1", :executables => true).uninstall
 
-    assert File.exist?(plugin_path), 'plugin removed when old version uninstalled'
-    assert_match %r{\Arequire.*a-3/lib/rubygems_plugin\.rb}, File.read(plugin_path), 'old version uninstalled, but plugin updated'
+    assert File.exist?(plugin_path), "plugin removed when old version uninstalled"
+    assert_match %r{\Arequire.*a-3/lib/rubygems_plugin\.rb}, File.read(plugin_path), "old version uninstalled, but plugin updated"
 
-    Gem::Uninstaller.new('a', version: '3', :executables => true).uninstall
+    Gem::Uninstaller.new("a", version: "3", :executables => true).uninstall
 
-    assert File.exist?(plugin_path), 'plugin removed when old version uninstalled and another version with plugin still present'
-    assert_match %r{\Arequire.*a-2/lib/rubygems_plugin\.rb}, File.read(plugin_path), 'latest version uninstalled, but plugin not updated to previous version'
+    assert File.exist?(plugin_path), "plugin removed when old version uninstalled and another version with plugin still present"
+    assert_match %r{\Arequire.*a-2/lib/rubygems_plugin\.rb}, File.read(plugin_path), "latest version uninstalled, but plugin not updated to previous version"
 
-    Gem::Uninstaller.new('a', version: '2', :executables => true).uninstall
+    Gem::Uninstaller.new("a", version: "2", :executables => true).uninstall
 
-    refute File.exist?(plugin_path), 'last version uninstalled, but plugin still present'
+    refute File.exist?(plugin_path), "last version uninstalled, but plugin still present"
   end
 end
