@@ -121,6 +121,15 @@ class TestSocket < Test::Unit::TestCase
     }
   end
 
+  def test_ip_address_list_include_localhost
+    begin
+      list = Socket.ip_address_list
+    rescue NotImplementedError
+      return
+    end
+    assert_includes list.map(&:ip_address), Addrinfo.tcp("localhost", 0).ip_address
+  end
+
   def test_tcp
     TCPServer.open(0) {|serv|
       addr = serv.connect_address
@@ -154,11 +163,19 @@ class TestSocket < Test::Unit::TestCase
   def random_port
     # IANA suggests dynamic port for 49152 to 65535
     # http://www.iana.org/assignments/port-numbers
-    49152 + rand(65535-49152+1)
+    case RUBY_PLATFORM
+    when /mingw|mswin/
+      rand(50000..65535)
+    else
+      rand(49152..65535)
+    end
   end
 
   def errors_addrinuse
-    [Errno::EADDRINUSE]
+    errs = [Errno::EADDRINUSE]
+    # MinGW fails with "Errno::EACCES: Permission denied - bind(2) for 0.0.0.0:49721"
+    errs << Errno::EACCES if /mingw/ =~ RUBY_PLATFORM
+    errs
   end
 
   def test_tcp_server_sockets
@@ -326,7 +343,7 @@ class TestSocket < Test::Unit::TestCase
     begin
       ifaddrs = Socket.getifaddrs
     rescue NotImplementedError
-      skip "Socket.getifaddrs not implemented"
+      omit "Socket.getifaddrs not implemented"
     end
 
     ifconfig = nil
@@ -380,11 +397,10 @@ class TestSocket < Test::Unit::TestCase
             in6_ifreq = [ifr_name,ai.to_sockaddr].pack('a16A*')
             s.ioctl(ulSIOCGIFFLAGS, in6_ifreq)
             next true if in6_ifreq.unpack('A16L1').last & ulIFF_POINTOPOINT != 0
-          else
-            ifconfig ||= `/sbin/ifconfig`
-            next true if ifconfig.scan(/^(\w+):(.*(?:\n\t.*)*)/).find do|ifname, value|
-              value.include?(ai.ip_address) && value.include?('POINTOPOINT')
-            end
+          end
+          ifconfig ||= `/sbin/ifconfig`
+          next true if ifconfig.scan(/^(\w+):(.*(?:\n\t.*)*)/).find do |_ifname, value|
+            value.include?(ai.ip_address) && value.include?('POINTOPOINT')
           end
         end
         false
@@ -421,7 +437,10 @@ class TestSocket < Test::Unit::TestCase
         }
       rescue NotImplementedError, Errno::ENOSYS
         skipped = true
-        skip "need sendmsg and recvmsg: #{$!}"
+        omit "need sendmsg and recvmsg: #{$!}"
+      rescue RuntimeError
+        skipped = true
+        omit "UDP server is no response: #{$!}"
       ensure
         if th
           if skipped
@@ -467,7 +486,8 @@ class TestSocket < Test::Unit::TestCase
         end while IO.select([r], nil, nil, 0.1).nil?
         n
       end
-      assert_equal([[s1],[],[]], IO.select([s1], nil, nil, 30))
+      timeout = (defined?(RubyVM::MJIT) && RubyVM::MJIT.enabled? ? 120 : 30) # for --jit-wait
+      assert_equal([[s1],[],[]], IO.select([s1], nil, nil, timeout))
       msg, _, _, stamp = s1.recvmsg
       assert_equal("a", msg)
       assert(stamp.cmsg_is?(:SOCKET, type))
@@ -563,7 +583,7 @@ class TestSocket < Test::Unit::TestCase
   ensure
     serv_thread.value.close
     server.close
-  end
+  end unless RUBY_PLATFORM.include?("freebsd")
 
   def test_connect_timeout
     host = "127.0.0.1"
@@ -736,6 +756,7 @@ class TestSocket < Test::Unit::TestCase
     ret, addr, rflags = s1.recvmsg(10, 0)
     assert_equal "b" * 10, ret
     assert_equal Socket::MSG_TRUNC, rflags & Socket::MSG_TRUNC if !rflags.nil?
+    addr
   ensure
     s1.close
     s2.close
